@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-2014 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1993-2021 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -920,6 +920,11 @@ void   DeleteToEOL()
    DeleteBlock(0,LineEnd(Offset())-Offset());
 }
 
+void   DeleteToBOL()
+{
+   DeleteBlock(Offset()-LineBegin(Offset()),0);
+}
+
 offs   NextLine(offs ptr)
 {
    char eol=EolStr[EolSize-1];
@@ -1277,3 +1282,63 @@ bool BlockEqAt(offs o,const char *s,int len)
    return !memcmp(buffer+o,s,len1)
        && !memcmp(buffer+GapSize,s+len1,len-len1);
 }
+
+offs FindMatch(const char op)
+{
+   byte cl;
+   int dir;
+   offs ptr=CurrentPos;
+   int level = 0;
+
+   switch(op)
+   {
+   case '[':
+      cl = ']';
+      dir = 1;
+      break;
+   case ']':
+      cl = '[';
+      dir = -1;
+      break;
+   case '{':
+      cl = '}';
+      dir = 1;
+      break;
+   case '}':
+      cl = '{';
+      dir = -1;
+      break;
+   case '(':
+      cl = ')';
+      dir = 1;
+      break;
+   case ')':
+      cl = '(';
+      dir = -1;
+      break;
+   case '<':
+      cl = '>';
+      dir = 1;
+      break;
+   case '>':
+      cl = '<';
+      dir = -1;
+      break;
+   default:
+      return -2;
+   }
+   while(!((dir>0)?EofAt(ptr):BofAt(ptr)))
+   {
+      ptr+=dir;
+      if(CharAt(ptr)==op)
+         level++;
+      else if(CharAt(ptr)==cl)
+      {
+         if(level==0)
+            return ptr;
+         else
+            level--;
+      }
+   }
+   return -1;
+}
